package mezz.jei.library.plugins.vanilla.brewing;

import mezz.jei.api.ingredients.IIngredientHelper;
import mezz.jei.api.ingredients.subtypes.UidContext;
import mezz.jei.core.collect.SetMultiMap;
import net.minecraft.class_1799;
import net.minecraft.class_1802;
import net.minecraft.class_1844;
import net.minecraft.class_1847;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class BrewingRecipeUtil {
	public static final class_1799 POTION = new class_1799(class_1802.field_8574);
	public static final class_1799 WATER_BOTTLE = class_1844.method_8061(POTION.method_7972(), class_1847.field_8991);

	private final Map<String, Integer> brewingStepCache = new HashMap<>(); // output potion -> brewing steps
	private final SetMultiMap<String, String> potionMap = new SetMultiMap<>(); // output potion -> input potions
	private final IIngredientHelper<class_1799> itemStackHelper;

	public BrewingRecipeUtil(IIngredientHelper<class_1799> itemStackHelper) {
		this.itemStackHelper = itemStackHelper;
		clearCache();
	}

	public void addRecipe(List<class_1799> inputPotions, class_1799 outputPotion) {
		String potionOutputUid = itemStackHelper.getUniqueId(outputPotion, UidContext.Recipe);
		for (class_1799 inputPotion : inputPotions) {
			String potionInputUid = itemStackHelper.getUniqueId(inputPotion, UidContext.Recipe);
			potionMap.put(potionOutputUid, potionInputUid);
		}
		clearCache();
	}

	public int getBrewingSteps(class_1799 outputPotion) {
		String potionInputUid = itemStackHelper.getUniqueId(outputPotion, UidContext.Recipe);
		return getBrewingSteps(potionInputUid, new HashSet<>());
	}

	private void clearCache() {
		if (brewingStepCache.size() != 1) {
			brewingStepCache.clear();
			String waterBottleUid = itemStackHelper.getUniqueId(WATER_BOTTLE, UidContext.Recipe);
			brewingStepCache.put(waterBottleUid, 0);
		}
	}

	private int getBrewingSteps(String potionOutputUid, Set<String> previousSteps) {
		Integer cachedBrewingSteps = brewingStepCache.get(potionOutputUid);
		if (cachedBrewingSteps != null) {
			return cachedBrewingSteps;
		}

		if (!previousSteps.add(potionOutputUid)) {
			return Integer.MAX_VALUE;
		}

		Collection<String> prevPotions = potionMap.get(potionOutputUid);
		int minPrevSteps = prevPotions.stream()
			.mapToInt(prevPotion -> getBrewingSteps(prevPotion, previousSteps))
			.min()
			.orElse(Integer.MAX_VALUE);

		int brewingSteps = minPrevSteps == Integer.MAX_VALUE ? Integer.MAX_VALUE : minPrevSteps + 1;
		brewingStepCache.put(potionOutputUid, brewingSteps);
		return brewingSteps;
	}
}
